<?php

namespace Sit\Destination\Providers;

use Illuminate\Support\ServiceProvider;

class DestinationServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'destination');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'destination');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerDestination();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerDestination()
    {
        $this->app->bind('destination', function($app) {
            return new Destination($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Destination', 'Lavalite\Destination\Facades\Destination');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('sit.destination', function ($app) {
            return $this->app->make('Sit\Destination\Destination');
        });

        // Bind Destination to repository
        $this->app->bind(
            'Sit\Destination\Interfaces\DestinationRepositoryInterface',
            \Sit\Destination\Repositories\Eloquent\DestinationRepository::class
        );        // Bind DestinationCategory to repository
        $this->app->bind(
            'Sit\Destination\Interfaces\DestinationCategoryRepositoryInterface',
            \Sit\Destination\Repositories\Eloquent\DestinationCategoryRepository::class
        );

        $this->app->register(\Sit\Destination\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Sit\Destination\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and destination's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/destination.php', 'destination'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeDestination::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['sit.destination'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('sit/destination.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/destination')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/destination')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
